import styled from '@emotion/styled';
import AutoFixHighIcon from '@mui/icons-material/AutoFixHigh';
import ContentCopy from '@mui/icons-material/ContentCopy';
import ContentPaste from '@mui/icons-material/ContentPaste';
import DeleteForever from '@mui/icons-material/DeleteForever';
import Alert from '@mui/material/Alert';
import Divider from '@mui/material/Divider';
import IconButton from '@mui/material/IconButton';
import ListItemIcon from '@mui/material/ListItemIcon';
import ListItemText from '@mui/material/ListItemText';
import Menu from '@mui/material/Menu';
import MenuItem from '@mui/material/MenuItem';
import Snackbar from '@mui/material/Snackbar';
import { useState } from 'react';
import useToast from './hooks/useToast';

const Z_INDEX_MAX = 2 ** 31 - 1;

const CustomButton = styled(IconButton)`
  position: fixed;
  right: 20px;
  bottom: 20px;
  z-index: ${Z_INDEX_MAX - 1};
  opacity: 0.75;
  transition: all 225ms linear;
  background-color: #c2e0ff;
  &:hover {
    opacity: 1;
    background-color: #99ccf3;
  }
`;

const CustomMenu = styled(Menu)`
  z-index: ${Z_INDEX_MAX};
`;

const App = () => {
  const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null);
  const menuOpen = Boolean(anchorEl);

  const {
    open: toastOpen,
    message: toastMessage,
    severity: toastSeverity,
    handleOpen: handleToastOpen,
    handleClose: handleToastClose,
  } = useToast();

  const handleMenuOpen = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };
  const handleMenuClose = () => {
    setAnchorEl(null);
  };

  const copyToClipboard = (text: string) => {
    GM_setClipboard(text);
  };

  const getCookie = () =>
    new Promise<TampermonkeyBeta.Cookie[]>((resolve, reject) => {
      GM_cookie('list', {}, (cookie, error) => {
        if (error) {
          reject(error);
        } else {
          resolve(cookie);
        }
      });
    });

  const setCookie = (cookieObj: TampermonkeyBeta.Cookie) =>
    new Promise<TampermonkeyBeta.Cookie[]>((resolve, reject) => {
      GM_cookie('set', cookieObj, (cookie, error) => {
        if (error) {
          reject(error);
        } else {
          resolve(cookie);
        }
      });
    });

  const deleteCookie = async () => {
    const currentCookie = await getCookie();
    if (currentCookie.length === 0) {
      return;
    }
    return new Promise((resolve, reject) => {
      currentCookie.forEach((cookieRecord) => {
        GM_cookie('delete', { name: cookieRecord.name }, (cookie, error) => {
          if (error) {
            reject(error);
          } else {
            resolve(cookie);
          }
        });
      });
    });
  };

  const getSession = async () => {
    try {
      const cookie = await getCookie();
      const exportSessionData = JSON.stringify(cookie);
      copyToClipboard(exportSessionData);
      handleToastOpen('Session 数据已复制到剪贴板', 'success');
    } catch (error) {
      handleToastOpen('获取 Cookie 失败', 'error');
    }
  };

  const setSession = () => {
    try {
      const importSessionData: string =
        prompt('在此处粘贴你的 Session 数据', '[{}]') || '[{}]';
      const cookie = JSON.parse(importSessionData) as TampermonkeyBeta.Cookie[];
      cookie.forEach(async (item) => {
        await setCookie(item);
      });
      handleToastOpen('Session 数据已导入', 'success');
    } catch (error) {
      handleToastOpen('导入 Cookie 失败', 'error');
    }
  };

  const deleteSession = async () => {
    try {
      await deleteCookie();
      handleToastOpen('Session 数据已删除', 'success');
    } catch (error) {
      handleToastOpen('删除 Cookie 失败', 'error');
    }
  };

  const handleGetSession = () => {
    getSession();
    handleMenuClose();
  };

  const handleSetSession = () => {
    setSession();
    handleMenuClose();
  };

  const handleDeleteSession = () => {
    deleteSession();
    handleMenuClose();
  };

  return (
    <>
      {/* 工具按钮 */}
      <CustomButton
        color="primary"
        id="custom-button"
        aria-controls={menuOpen ? 'custom-menu' : undefined}
        aria-haspopup="true"
        aria-expanded={menuOpen ? 'true' : undefined}
        onClick={handleMenuOpen}
      >
        <AutoFixHighIcon />
      </CustomButton>

      {/* 工具菜单 */}
      <CustomMenu
        id="custom-menu"
        anchorEl={anchorEl}
        open={menuOpen}
        onClose={handleMenuClose}
        MenuListProps={{
          'aria-labelledby': 'custom-button',
        }}
      >
        <MenuItem onClick={handleGetSession}>
          <ListItemIcon>
            <ContentCopy fontSize="small" />
          </ListItemIcon>
          <ListItemText>导出 Session 数据</ListItemText>
        </MenuItem>

        <MenuItem onClick={handleSetSession}>
          <ListItemIcon>
            <ContentPaste fontSize="small" />
          </ListItemIcon>
          <ListItemText>导入 Session 数据</ListItemText>
        </MenuItem>

        <Divider />

        <MenuItem onClick={handleDeleteSession}>
          <ListItemIcon>
            <DeleteForever fontSize="small" />
          </ListItemIcon>
          <ListItemText>清除 Session 数据</ListItemText>
        </MenuItem>
      </CustomMenu>

      {/* 提示 */}
      <Snackbar
        anchorOrigin={{ vertical: 'bottom', horizontal: 'center' }}
        open={toastOpen}
        autoHideDuration={2000}
        onClose={handleToastClose}
      >
        <Alert
          onClose={handleToastClose}
          severity={toastSeverity}
          sx={{ width: '100%' }}
        >
          {toastMessage}
        </Alert>
      </Snackbar>
    </>
  );
};

export default App;
